type ligne = { num   : int;    (* numero de la ligne *)
               y     : float;}  (* ordonnee de la ligne *)

type agent = int -> ligne -> int * int list

exception Invalid_args

(* dimension du plan *)
let x_min = (-2.0)
let x_max = (1.0)
let y_min = (-1.5)
let y_max = (1.5)
let step  = (0.005)

let n_x   = int_of_float ((x_max -. x_min) /. step) 

(* conditions de convergence/divergence *)
let break_value_d  = 500.
let max_iterations = 2000

(* suite complexe pour le calcul de l'espace de mandelbrot *)
let rec suite base var i =
  if i > max_iterations then
    i
  else
    let res = Complex.add (Complex.mul var var) base in
    if Complex.norm2 res >= break_value_d then
      i
    else
      suite base res (i+1)

(* teste la convergence de la suite pour le point (x,y) *)    
let process_point x y =
  suite
    { Complex.re = x;
      Complex.im = y;}
    Complex.zero 0


let process_line line =
  let rec fill_list x l =
    if x >= n_x then l
    else
      fill_list 
        (x+1)
        ((process_point (x_min +. (float_of_int x) *. step) line.y) :: l)
  in
  List.rev (fill_list 0 [])

 
