type ligne = { num   : int;    (* numero de la ligne *)
               y     : float;}  (* ordonnee de la ligne *)

type agent = int -> ligne -> int * int list

exception Invalid_args

(* dimension du plan *)
let x_min = (-2.0)
let x_max = (1.0)
let y_min = (-1.5)
let y_max = (1.5)
let step  = (0.005)

let n_x   = int_of_float ((x_max -. x_min) /. step) 

(* conditions de convergence/divergence *)
let break_value_d  = 500.
let max_iterations = 2000

(* suite complexe pour le calcul de l'espace de mandelbrot *)
let rec suite base var i =
  if i > max_iterations then
    i
  else
    let res = Complex.add (Complex.mul var var) base in
    if Complex.norm2 res >= break_value_d then
      i
    else
      suite base res (i+1)

(* teste la convergence de la suite pour le point (x,y) *)    
let process_point x y =
  suite
    { Complex.re = x;
      Complex.im = y;}
    Complex.zero 0


let process_line line =
  let rec fill_list x l =
    if x >= n_x then l
    else
      fill_list 
        (x+1)
        ((process_point (x_min +. (float_of_int x) *. step) line.y) :: l)
  in
  List.rev (fill_list 0 [])

 
let nb_lignes = int_of_float ((y_max -. y_min) /. step)
let nb_worker = 8

let gen_ligne n =
  {num = n; y = y_min +. (float_of_int n) *. step}

let draw_line line n =
  match line with 
    | None -> ()
    | Some l ->
	begin
	  if n >= nb_lignes then ()
	  else
	    begin
	      let rec draw_list l i =
		match l with
		  | [] -> ()
		  | hd :: tl ->
		      let c = hd * 255 / max_iterations in
			draw_list tl (i + 1)
	      in
		draw_list l 0;
		(*Graphics.synchronize()*)
	    end
	end

let rec worker n res =
  if n > nb_lignes - 8 then ()
  else
    begin
      res := process_line (gen_ligne n) :: !res;
      print_endline (string_of_int n);
      worker (n + 8) res
    end


(* et c'est la que ca devient crade... *)

(* stockage des résultats *)
let r0 = ref []
let r1 = ref []
let r2 = ref []
let r3 = ref []
let r4 = ref []
let r5 = ref []
let r6 = ref []
let r7 = ref []

let pop r = match !r with
  | hd :: tl -> r := tl; Some hd
  | [] -> None

let draw_all () =
  let rec f n =
    if n >= nb_lignes then ()
    else
      begin
	draw_line (pop r7) n;
	draw_line (pop r6) (n+1);
	draw_line (pop r5) (n+2);
	draw_line (pop r4) (n+3);
	draw_line (pop r3) (n+4);
	draw_line (pop r2) (n+5);
	draw_line (pop r1) (n+6);
	draw_line (pop r0) (n+7);
	f (n+8)
      end
  in
    f 0

let _ =
  let t0 = Thread.create (worker 0) r0 in
  let t1 = Thread.create (worker 1) r1 in
  let t2 = Thread.create (worker 2) r2 in
  let t3 = Thread.create (worker 3) r3 in
  let t4 = Thread.create (worker 4) r4 in
  let t5 = Thread.create (worker 5) r5 in
  let t6 = Thread.create (worker 6) r6 in
  let t7 = Thread.create (worker 7) r7 in
    Thread.join t0;
    print_endline "thread 0 termine";
    Thread.join t1;
    print_endline "thread 1 termine";
    Thread.join t2;
    print_endline "thread 2 termine";
    Thread.join t3;
    print_endline "thread 3 termine";
    Thread.join t4;
    print_endline "thread 4 termine";
    Thread.join t5;
    print_endline "thread 5 termine";
    Thread.join t6;
    print_endline "thread 6 termine";
    Thread.join t7;
    print_endline "thread 7 termine";
draw_all ();
    print_endline "affichage reussit";
    ignore (input_char stdin)
    
